/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: common.c,v 1.3 2004/12/04 13:09:33 lordjaxom Exp $
 */

#include "common.h"
#include "setup.h"
#include <vdr/recording.h>
#include <sys/types.h>
#include <dirent.h>
#include <glob.h>

const char *BurnStates[__STATE_COUNT__] = { "List", "Status" };
const char *DiskTypes[__TYPE_COUNT__]   = { "DVD with menus", "Archive disk" };
const char *StoreModes[__STORE_COUNT__] = { "Create ISO only", "Write to Disc", "Create ISO and write" };
const char *BurnModes[__MODE_COUNT__]   = { "Shrink", "Distribute" };

const char *TitleChars = "abcdefghijklmnopqrstuvwxyz"
                         "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
												 ":- ";

uint64_t ScanRecordingSize(const char *FileName) {
	uint64_t size = 0;
	int count = 0;
	int res = 0;
	while (res == 0) {
		char *path;
		struct stat64 buf;
		asprintf(&path, "%s/%03d.vdr", FileName, count + 1);
		if ((res = stat64(path, &buf)) == 0) {
			size += buf.st_size;
			++count;
		}	
		free(path);
	}

	if (count == 0)
		esyslog("BURN: No datafiles in %s (this shouldn't happen!)", FileName);
	return size;
}

int ScanPageCount(const char *Path) {
	glob_t globbuf;
	char *path;
	int res;

	asprintf(&path, "%s/menu-bg-*.png", Path);
	if (glob(path, 0, NULL, &globbuf) != 0) {
		esyslog("BURN: Error while scanning %s: %m", path);
		free(path);
		return -1;
	}

	res = globbuf.gl_pathc;
	globfree(&globbuf);
	free(path);
	return res;
}

uint64_t FileSize(const char *FileName) {
	int res;
	struct stat64 buf;
	if ((res = stat64(FileName, &buf)) == 0)
		return buf.st_size;
	return 0;
}

const char *FileType(const char *FileName) {
	int len;
	return (len = strlen(FileName)) > 3 ? FileName + len - 3 : "";
}

const char *ProgressBar(double Current, double Total, int Length) {
	static char *buffer = NULL;
	int l;
	free(buffer);
	if (Length < 2)
		return "";
	buffer = MALLOC(char, Length + 1);
	buffer[0] = '[';
	if (Current > Total)
		Current = Total;
	l = (int)(Current * (Length - 2) / Total);
	memset(buffer + 1, '#', l);
	memset(buffer + l + 1, '-', Length - 2 - l);
	buffer[Length - 1] = ']';
	buffer[Length] = '\0';
	return buffer;
}

const tTrackInfo *FindTitleTracks(const char *Path) {
	static tTrackInfo tracks[MAXTRACKS];
	glob_t globbuf;
	char *path;
	int pos = 0;

	// initialize array
	for (int i = 0; i < MAXTRACKS; ++i) {
		tracks[i].Filename = NULL;
		tracks[i].Type = ttUnknown;
		tracks[i].Language = -1;
	}

	asprintf(&path, "%s/*.mpv", Path);
	if (glob(path, 0, NULL, &globbuf) != 0 || globbuf.gl_pathc != 1) {
		globfree(&globbuf);
		free(path);
		asprintf(&path, "%s/*.m2v", Path);
		if (glob(path, 0, NULL, &globbuf) != 0 || globbuf.gl_pathc != 1) {
			globfree(&globbuf);
			free(path);
			return NULL;
		}
	}
	tracks[pos].Filename = strdup(globbuf.gl_pathv[0]);
	tracks[pos].Type = ttVideo;
	tracks[pos].Language = Setup.OSDLanguage;
	++pos;
	
	globfree(&globbuf);
	free(path);

	asprintf(&path, "%s/*.ac3", Path);
	if (glob(path, 0, NULL, &globbuf) == 0) {
		for (int i = 0; i < (int)globbuf.gl_pathc; ++i) {
			tracks[pos].Filename = strdup(globbuf.gl_pathv[i]);
			tracks[pos].Type = ttAudioAC3;
			tracks[pos].Language = Setup.OSDLanguage;
			++pos;
		}
	}
	globfree(&globbuf);
	free(path);

	asprintf(&path, "%s/*.mpa", Path);
	if (glob(path, 0, NULL, &globbuf) == 0) {
		for (int i = 0; i < (int)globbuf.gl_pathc; ++i) {
			tracks[pos].Filename = strdup(globbuf.gl_pathv[i]);
			tracks[pos].Type = ttAudioAC3;
			tracks[pos].Language = Setup.OSDLanguage;
			++pos;
		}
	}
	globfree(&globbuf);
	free(path);

	return tracks;
}

char *EscapeLatin1toUTF8(const char *Text) {
	int len = strlen(Text);
	uchar *in = (uchar*)Text;
	char *utf = MALLOC(char, len * 2 + 1);
	uchar *out = (uchar*)utf;

	for (; *in; ++in) {
		if (BurnSetup.UseIso2UTF && *in >= 0x80) {
			*out++ = ((*in >> 6) & 0x1F) | 0xC0;
			*out++ = (*in & 0x3F) | 0x80;
		/*} else if (*in == '"') {
			*out++ = '\\';
			*out++ = '"';*/
		} else
			*out++ = *in;
	}
	*out = 0;
	return utf;
}
