/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: common.h,v 1.3 2004/12/04 13:09:33 lordjaxom Exp $
 */

#ifndef VDR_BURN_COMMON_H
#define VDR_BURN_COMMON_H

#include <stdint.h>
#include <stdlib.h>
#include <vdr/tools.h>

#define BURNMARKFILE "burnmark.vdr"

#define JOBNAMELEN 255
#define MAXTRACKS 9 // the maximum tcmplex-panteltje can handle
#define MAXPAGETITLES 8
#define DVDSIZE 4462 // from vdrconvert, minus some reserve
// #define DVDSIZE 200

#ifdef DEBUG
#	include <stdio.h>
#	define Dprintf(x...) fprintf(stderr, x)
#else
#	define Dprintf(x...)
#endif

#define strn1cmp(a,b) strncmp(a,b,strlen(b))

enum eBurnState {
	bsRecordings,
	bsStatus,
	__STATE_COUNT__
};

enum eDiskType {
	dtDVD,
	__TYPE_COUNT_REQUANT__,
	dtVDR = __TYPE_COUNT_REQUANT__,
	__TYPE_COUNT__
};

enum eStoreMode {
	smCreateISO,
	smBurnISO,
	smCreateBurn,
#define __STORE_COUNT__ smCreateBurn + 1
};

enum eBurnMode {
	bmShrink,
	bmDistribute,
	__MODE_COUNT__
};
	
enum eReturnStatus {
	rsNone,
	rsOk,
	rsError
};

enum eNextStep {
	nsContinue,
	nsNext,
	nsRetry
};
	
enum eTrackType {
	ttUnknown,
	ttVideo,
	ttAudioMP2,
	ttAudioAC3
};

struct tTrackInfo {
	char *Filename;
	eTrackType Type;
	int Language;
	tTrackInfo(void) { Filename = NULL; Type = ttUnknown; Language = -1; }
	~tTrackInfo() { free(Filename); }
};

typedef void(*BurnProc)(void *Data);
	
extern const char *BurnStates[__STATE_COUNT__];
extern const char *DiskTypes[__TYPE_COUNT__];
extern const char *StoreModes[__STORE_COUNT__];
extern const char *BurnModes[__MODE_COUNT__];

extern const char *TitleChars;

class cRecording;

uint64_t ScanRecordingSize(const char *FileName);
int ScanPageCount(const char *Path);
uint64_t FileSize(const char *FileName);
const char *FileType(const char *FileName);
const char *ProgressBar(double Current, double Total, int Length = 20);

const tTrackInfo *FindTitleTracks(const char *Path);

char *EscapeLatin1toUTF8(const char *Text);

#endif // VDR_BURN_COMMON_H
