/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: jobs.c,v 1.3 2004/12/02 15:02:52 lordjaxom Exp $
 */

#include "jobs.h"
#include "setup.h"
#include "process-dvd.h"
#include "process-vdr.h"
#include <vdr/recording.h>

// --- cBurnRecording ---------------------------------------------------------

cBurnRecording::cBurnRecording(cRecording *Recording, time_t Timestamp) {
	char *ptr, *tmp;
	mFileName = strdup(Recording->FileName());
	strncpy(mName, Recording->Name(), JOBNAMELEN);
	mSummary = (Recording->Summary() ? strdup(Recording->Summary()) : NULL);
	mTitle = strdup(Recording->Title('\t'));
	mTempDir = NULL;
	mTimestamp = Timestamp;
	mNumber = -1;
	mSizeMB = 0;

	tmp = mName;
	do {
		ptr = tmp;
		while (strchr("@%", *ptr) != NULL)
			++ptr;
		memmove(tmp, ptr, strlen(ptr) + 1);
		if ((tmp = strchr(tmp, '~')) != NULL)
			++tmp;
	} while (tmp != NULL);

	ptr = mTitle;
	while ((tmp = strchr(ptr + 1, '\t')) != NULL) {
		*tmp = ' ';
		ptr = tmp;
	}
	*ptr = '\0';
}

cBurnRecording::cBurnRecording(const cBurnRecording &Recording) {
	mFileName = strdup(Recording.mFileName);
	strncpy(mName, Recording.mName, JOBNAMELEN);
	mSummary = (Recording.mSummary ? strdup(Recording.mSummary) : NULL);
	mTitle = strdup(Recording.mTitle);
	mTempDir = (Recording.mTempDir ? strdup(Recording.mTempDir) : NULL);
	mTimestamp = Recording.mTimestamp;
	mNumber = Recording.mNumber;
	mSizeMB = Recording.mSizeMB;
}

cBurnRecording::~cBurnRecording(void) {
	free(mFileName);
	free(mSummary);
	free(mTitle);
	free(mTempDir);
}

void cBurnRecording::SetTempDir(const char *TempDir) {
	free(mTempDir);
	asprintf(&mTempDir, "%s/VDRSYNC.%d", TempDir, mNumber);
}

double cBurnRecording::SizeMB(void) {
	if (mSizeMB == 0)
		mSizeMB = ScanRecordingSize(mFileName) / 1024.f / 1024.f;
	return mSizeMB;
}

const tTrackInfo *cBurnRecording::GetTracks(void) {
	if (mTracks[0].Type == ttUnknown) {
		const tTrackInfo *tracks = FindTitleTracks(mTempDir);
		if (tracks == NULL)
			return NULL;
		memcpy(mTracks, tracks, sizeof(tTrackInfo) * MAXTRACKS);
	}
	return mTracks;
}
	
// --- cBurnJob ---------------------------------------------------------------

cBurnJob::cBurnJob(void) {
	mTempDir = NULL;
	mTitle[0] = '\0';
	mDiskType = dtDVD;
	if (DVDdev == NULL)
		mStoreMode = smCreateISO;
	else if (ISOdir == NULL)
		mStoreMode = smBurnISO;
	else
		mStoreMode = smCreateBurn;
	mBurnMode = bmShrink;
	mProcess = NULL;
}

cBurnJob::~cBurnJob() {
	delete mProcess;
	free(mTempDir);
}

void cBurnJob::Start(void) {
	if (mProcess) {
		esyslog("BURN: Process already running (This shouldn't happen!)");
		return;
	}

	switch (mDiskType) {
	case dtDVD:
		mProcess = new cBurnProcessDVD(this);
		break;

	case dtVDR:
		mProcess = new cBurnProcessVDR(this);
		break;

	default:
		break;
	};
}

void cBurnJob::Stop(void) {
	if (mProcess)
		mProcess->Stop();
}

void cBurnJob::Add(cBurnRecording *Object) {
	cBurnRecording *before = First();
	Object->SetNumber(Count());
	if (mTitle[0] == '\0')
		strncpy(mTitle, Object->Name(), JOBNAMELEN);
	while (before && before->Timestamp() < Object->Timestamp())
		before = Next(before);
	if (before)
		cListBase::Ins(Object, before);
	else
		cListBase::Add(Object);
}

void cBurnJob::SetTempDir(const char *TempDir) {
	for (cBurnRecording *rec = First(); rec; rec = Next(rec))
		rec->SetTempDir(TempDir);
	mTempDir = strdup(TempDir);
}

cBurnRecording *cBurnJob::GetByFileName(const char *FileName) {
	for (cBurnRecording *rec = First(); rec; rec = Next(rec)) {
		if (strcmp(rec->FileName(), FileName) == 0)
			return rec;
	}
	return NULL;
}
