/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: process-vdr.c,v 1.2 2004/12/05 18:50:34 lordjaxom Exp $
 */

#include "process-vdr.h"
#include "jobs.h"
#include "menuburn.h"
#include "i18n.h"
#include "setup.h"
#include <vdr/tools.h>
#include <vdr/interface.h>
#include <vdr/videodir.h>
#include <stdlib.h>

cBurnProcessVDR::cBurnProcessVDR(cBurnJob *Job):
		cBurnProcess(__VDR_COUNT__) {
	mJob = Job;

	mJob->SetTempDir(TempDir());
	SetLogfile("%s/vdr.log", TempDir());
	Start();
}

cBurnProcessVDR::~cBurnProcessVDR() {
}

void cBurnProcessVDR::Process(int Step, int SubStep, const char *Line) {
	float fdone;

	switch ((eStepVDR)Step) {
	case svGrowisofs:
		if (sscanf(Line, "%f%% done, estimate finish", &fdone) == 1) {
			SetBurning(true, (int)fdone);
			SetProgress(1, (int)fdone);
		}
		break;

	default:
		break;
	}
}

bool cBurnProcessVDR::Continue(int Step, int SubStep) {
	cBurnRecording *rec;
	switch ((eStepVDR)Step) {
	case svGrowisofs:
		if (SubStep == 0) {
			char *cmd, *tmp, *path;

			switch (mJob->StoreMode()) {
			case smCreateISO:
				asprintf(&cmd, "mkisofs "
						"-V VDR_BURN -r -J -l -allow-lowercase -allow-multidot "
						"-graft-points");
				for (rec = mJob->First(); rec; rec = mJob->Next(rec)) {
					tmp = cmd;
					asprintf(&cmd,"%s '%s'='%s'", tmp, rec->FileName() + strlen(Datadir),
							rec->FileName());
					free(tmp);
				}
				tmp = cmd;
				asprintf(&cmd, "%s > '%s/%s.iso'", tmp, ISOdir, mJob->Title());
				free(tmp);
				break;

			case smBurnISO:
				asprintf(&cmd, "growisofs -use-the-force-luke=tty -Z %s "
						"-V VDR_BURN -r -J -l -allow-lowercase -allow-multidot "
						"-graft-points", DVDdev);
				for (rec = mJob->First(); rec; rec = mJob->Next(rec)) {
					char *tmp = cmd;
					asprintf(&cmd,"%s '%s'='%s'", tmp, rec->FileName() + strlen(Datadir),
							rec->FileName());
					free(tmp);
				}
				break;

			case smCreateBurn:
				asprintf(&path, "%s/fifo", TempDir());
				mkfifo(path, 0644);
				asprintf(&cmd, "growisofs -use-the-force-luke=tty -Z %s=%s | mkisofs "
						"-V VDR_BURN -r -J -l -allow-lowercase -allow-multidot "
						"-graft-points", DVDdev, path);
				for (rec = mJob->First(); rec; rec = mJob->Next(rec)) {
					tmp = cmd;
					asprintf(&cmd,"%s '%s'='%s'", tmp, rec->FileName() + strlen(Datadir),
							rec->FileName());
					free(tmp);
				}
				tmp = cmd;
				asprintf(&cmd, "%s | tee '%s/%s.iso' > %s", tmp, ISOdir, mJob->Title(), path);
				free(tmp);
				free(path);
				break;
			}

			SetBurning(true);
			SetCmdline(cmd);
			free(cmd);
		} else if (Status() == rsError) {
			Execute("eject %s", DVDdev);
			cBurnMenuBurn::Confirm(tr("Please insert an empty disk and press Ok"));
			SetNextStep(nsRetry);
		} else {
			SetBurning(false);
			SetNextStep();
		}
		break;

	default:
		break;
	}

	return true;
}

