/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: process.h,v 1.2 2004/12/02 15:02:52 lordjaxom Exp $
 */

#ifndef VDR_BURN_PROCESS_H
#define VDR_BURN_PROCESS_H

#include "common.h"
#include <vdr/thread.h>
#include <string.h>

class cBurnProcess: public cThread {
private:
	eReturnStatus mStatus;
	char *mCmdline;
	BurnProc mProcedure;
	void *mProcData;
	bool mActive;
	bool mBurning;
	int mProgress;
	int mBurnProgress;
	char *mTempDir;

	int mNumSteps;
	int mCurStep;
	int mSubStep;
	eNextStep mNextStep;

	FILE *mLogFile;

protected:
	bool HasNext(void);
	void PutLine(const char *Line);

	virtual void Action(void);
	virtual void Process(int Step, int SubStep, const char *Line) = 0;
	virtual bool Continue(int Step, int SubStep) = 0;

	void SetProgress(int SubSteps, int Percent);
	void SetProgress(int SubSteps, int Subsub, int Subsubs, int Percent);
	void SetCmdline(const char *Format, ...) 
			__attribute__ ((format (printf, 2, 3)));
	void SetProcedure(BurnProc Procedure, void *Data);
	void SetStatus(eReturnStatus Status) { mStatus = Status; }
	void SetNextStep(eNextStep Next = nsNext) { mNextStep = Next; }
	void SetBurning(bool Burning, int Progress = 0);
	void SetLogfile(const char *Format, ...)
			__attribute__ ((format (printf, 2, 3)));
	void MakeDirectory(const char *Format, ...)
			__attribute__ ((format (printf, 2, 3)));
	void Execute(const char *Format, ...)
			__attribute__ ((format (printf, 2, 3)));

public:
	cBurnProcess(int NumSteps);
	virtual ~cBurnProcess();
	
	virtual void Stop(void);

	int Progress(void) const { return mProgress; }
	eReturnStatus Status(void) const { return mStatus; }
	bool Burning(int &Progress) const;
	const char *TempDir(void) const { return mTempDir; }
};

inline void cBurnProcess::SetBurning(bool Burning, int Progress) { 
	mBurning = Burning; 
	mBurnProgress = Progress; 
}

inline void cBurnProcess::SetProcedure(BurnProc Procedure, void *Data) {
	mProcedure = Procedure;
	mProcData = Data;
}

inline bool cBurnProcess::Burning(int &Progress) const {
	if (mBurning)
		Progress = mBurnProgress;
	return mBurning;
}

#endif // VDR_BURN_PROCESS_H
